@@ -44,16 +44,23 @@ namespace Lum {
 
     bool VisitChildren(Visitor &visitor, bool onlyVisible);
 
-    void Add(Object* object);
-    void AddOptional(Object* object);
+    ButtonRow* Add(Object* object);
+    ButtonRow* AddOptional(Object* object);
 
     void CalcSize();
     void Layout();
 
-    static ButtonRow* CreateOk(Model::Action* ok);
-    static ButtonRow* CreateCancel(Model::Action* cancel);
-    static ButtonRow* CreateOkCancel(Model::Action* ok, Model::Action* cancel);
-    static ButtonRow* CreateClose(Model::Action* close);
+    static ButtonRow* Create(bool horizontalFlex=false, bool verticalFlex=false);
+    static ButtonRow* CreateOk(Model::Action* ok,
+                               bool horizontalFlex=false, bool verticalFlex=false);
+    static ButtonRow* CreateCancel(Model::Action* cancel,
+                                   bool horizontalFlex=false, bool verticalFlex=false);
+    static ButtonRow* CreateOkCancel(Model::Action* ok, Model::Action* cancel,
+                                     bool horizontalFlex=false, bool verticalFlex=false);
+    static ButtonRow* CreateClose(Model::Action* close,
+                                  bool horizontalFlex=false, bool verticalFlex=false);
+    static ButtonRow* CreateQuit(Model::Action* quit,
+                                 bool horizontalFlex=false, bool verticalFlex=false);
   };
 }
 
