@@ -48,15 +48,19 @@ namespace Lum {
       Desc             description;
       Model::ActionRef action;
       Type             type;
+      bool             opensDialog;
 
     public:
       Action(const Desc& description, Model::Action *action);
 
-      Action* SetType(Type type);
+      Action& SetType(Type type);
       Type GetType() const;
 
       Model::Action* GetAction() const;
       const Desc& GetDesc() const;
+
+      Action& SetOpensDialog();
+      bool GetOpensDialog() const;
     };
   }
 }
