@@ -60,10 +60,9 @@ namespace Lum {
     };
 
   private:
-    std::list<Entry> list;        // list of rows
-    size_t           labelWidth;  // Maximum width of labels
-    size_t           objectWidth; // Maximum width of objectss
-    bool             labelFlex;   // Resize the label
+    std::list<Entry> list;        //! list of rows
+    size_t           labelWidth;  //! Maximum width of labels
+    size_t           objectWidth; //! Maximum width of objectss
 
   public:
     Label();
@@ -75,8 +74,6 @@ namespace Lum {
     Label* AddLabel(const std::wstring& label, Object* object);
     Label* AddLabel(const std::wstring& label, const std::wstring& object);
 
-    void SetLabelFlex(bool flex);
-
     void CalcSize();
     void Layout();
 
