@@ -38,6 +38,9 @@ namespace Lum {
     public:
       Adjustment();
 
+      void On(bool notify=true);
+      void Off();
+
       bool IsValid() const;
       size_t GetTop() const;
       size_t GetBottom() const;
@@ -54,7 +57,9 @@ namespace Lum {
       SizeT* GetTotalModel() const;
 
       void IncTop();
+      void IncTop(size_t stepSize);
       void DecTop();
+      void DecTop(size_t stepSize);
       void PageBack();
       void PageForward();
 
