@@ -61,6 +61,7 @@ namespace Lum {
 
       public:
         Window(OS::Display* display);
+        ~Window();
 
         // setter
         void SetTitle(const std::wstring& name);
@@ -94,9 +95,6 @@ namespace Lum {
         void Disable();
 
         bool SendBalloonMessage(const std::wstring& text,unsigned long timeout);
-
-        bool IsDoubleClicked() const;
-        bool IsTrippleClicked() const;
       };
     }
   }
