@@ -57,7 +57,7 @@ namespace Lum {
     {
 
     public:
-      static const size_t colorCount = 14; //! Number of predefined colors
+      static const size_t colorCount = 16; //! Number of predefined colors
       static const size_t fontCount  =  2; //! Number of predefined fonts
       static const size_t fillCount  = 42; //! Number of predefined fills
       static const size_t frameCount =  7; //! Number of predefined frames
@@ -78,7 +78,9 @@ namespace Lum {
         tabTextColor                  = 10, //! Font color for a textual tab rider
         tabTextSelectColor            = 11, //! Font color of a selected textual tab rider
         editTextColor                 = 12, //! Font color for edit fields and similar
-        editTextFillColor             = 13  //! Font color for selected texts in edit fields and similar
+        editTextFillColor             = 13, //! Font color for selected texts in edit fields and similar
+        textSmartAColor               = 14, //! Back font color for smart text
+        textSmartBColor               = 15  //! Front font color for smart text
       };
 
       enum FillIndex {
