@@ -55,21 +55,23 @@ namespace Lum {
         imageDefault     =  2, //! Image for marking a default action (e.g. for a button)
         imageHelp        =  3, //! Image for marking a help action (e.g. for a button)
         imageClose       =  4, //! Image for closing widget
-        imageAdd         =  5, //! Image for adding somethng
+        imageAdd         =  5, //! Image for adding something
         imageRemove      =  6, //! Image for removing something
-        imageEdit        =  7, //! Image for editing something
-        imageSave        =  8, //! Image for saving something
-        imageMediaRecord =  9, //! Image for start recordin media playing
-        imageMediaPlay   = 10, //! Image for playing media playing
-        imageMediaPause  = 11, //! Image for pausing media playing
-        imageMediaStop   = 12  //! Image for stoping media playing
+        imageClear       =  7, //! Image for clearing something (like e.g. a string field)
+        imageEdit        =  8, //! Image for editing something
+        imageSave        =  9, //! Image for saving something
+        imageMediaRecord = 10, //! Image for start recordin media playing
+        imageMediaPlay   = 11, //! Image for playing media playing
+        imageMediaPause  = 12, //! Image for pausing media playing
+        imageMediaStop   = 13  //! Image for stoping media playing
       };
 
-      static const size_t imageCount = 13; //! Number of predefined stock images
+      static const size_t imageCount = 14; //! Number of predefined stock images
 
       enum ScrollKnobMode {
         scrollKnobModeSingle,
-        scrollKnobModeDouble
+        scrollKnobModeDouble,
+        scrollKnobModeNone
       };
 
       enum ToolbarPosition {
@@ -119,6 +121,11 @@ namespace Lum {
 
       virtual void PushUniqueFill(DrawInfo* draw, size_t index, size_t maximum) = 0;
       virtual void PopUniqueFill(DrawInfo* draw, size_t index, size_t maximum) = 0;
+
+      /**
+        Background is light (instead of dark).
+        */
+      virtual bool IsLightTheme() const = 0;
       //@}
 
       /**
@@ -138,6 +145,11 @@ namespace Lum {
       virtual size_t GetVerticalFuelBarMinWidth() const = 0;
       virtual size_t GetVerticalFuelBarMinHeight() const = 0;
 
+      /**
+        Return the width of the cursor depending on the font.
+        */
+      virtual size_t GetCursorWidth(OS::Font* font) const = 0;
+
       /**
         How many pixels must the mouse move before this is recognized as drag start?
       */
