@@ -108,7 +108,9 @@ namespace Lum {
         bool GetAlpha() const;
         unsigned long GetDrawCap() const;
 
-        void Draw(OS::DrawInfo *draw, int x, int y, size_t w, size_t h);
+        void Draw(OS::DrawInfo *draw, int x, int y);
+        void DrawScaled(OS::DrawInfo *draw, int x, int y, size_t w, size_t h);
+        void DrawStretched(OS::DrawInfo *draw, int x, int y, size_t w, size_t h);
       };
 
       class GtkStockImage : public OS::Image
@@ -132,7 +134,9 @@ namespace Lum {
         bool GetAlpha() const;
         unsigned long GetDrawCap() const;
 
-        void Draw(OS::DrawInfo *draw, int x, int y, size_t w, size_t h);
+        void Draw(OS::DrawInfo *draw, int x, int y);
+        void DrawScaled(OS::DrawInfo *draw, int x, int y, size_t w, size_t h);
+        void DrawStretched(OS::DrawInfo *draw, int x, int y, size_t w, size_t h);
       };
 
       class GtkFill : public Fill
@@ -159,8 +163,9 @@ namespace Lum {
         void Draw(OS::DrawInfo* draw, int x, int y, size_t w, size_t h);
       };
 
-      extern int GetWidgetStyleIntValue(GtkWidget *widget, const char* name);
       extern bool GetWidgetStyleBoolValue(GtkWidget *widget, const char* name);
+      extern int GetWidgetStyleIntValue(GtkWidget *widget, const char* name);
+      extern float GetWidgetStyleFloatValue(GtkWidget *widget, const char* name);
       extern void GetWidgetStyleBorderValue(GtkWidget *widget, const char* name,
                                             int& left, int&right, int &top, int& bottom);
 
@@ -187,6 +192,8 @@ namespace Lum {
         GtkTheme(OS::Display *display);
         ~GtkTheme();
 
+        bool IsLightTheme() const;
+
         bool HasFontSettings() const;
 
         std::wstring GetProportionalFontName() const;
