@@ -87,6 +87,24 @@ namespace Lum {
 
   typedef Lum::Base::Reference<Controller> ControllerRef;
 
+  /**
+    Abstract base class for handling input method protocol handling.
+
+    Classes that return an IMHandler via GetIMHandler() can participate
+    in IM handling.
+   */
+  class IMHandler
+  {
+  public:
+    virtual ~IMHandler();
+    virtual void GetData(std::wstring& data,
+                         size_t& cursorOffset) = 0;
+    virtual void SetData(const std::wstring& data) = 0;
+    virtual void SetCursorOffset(size_t cursorOffset) = 0;
+    virtual bool IsInEditingArea(OS::MouseEvent* event) const;
+  };
+
+
   /**
    \brief The base class for all graphical objects.
 
@@ -390,6 +408,7 @@ namespace Lum {
 
     virtual void SetRequestsKeyboard(bool request);
     virtual bool RequestsKeyboard() const;
+    virtual IMHandler* GetIMHandler();
 
     virtual Object* GetFocusFirst();
     virtual Object* GetFocusLast();
@@ -405,6 +424,13 @@ namespace Lum {
     virtual void LostFocus();
 
     virtual void SetLabelObject(Object* label);
+
+    /**
+      If true is returned, this control has implicitly a label (like
+      for example a button through its button label) and thus does not
+      need an extra label like a Label would offer it.
+      */
+    virtual bool HasImplicitLabel() const;
     //@}
 
     virtual bool VisitChildren(Visitor &visitor, bool onlyVisible);
@@ -479,6 +505,7 @@ namespace Lum {
       @name Mouse and keyboard input callbacks
     */
     //@{
+    virtual bool InterceptMouseEvents() const;
     virtual bool HandleMouseEvent(const OS::MouseEvent& event);
     virtual bool HandleKeyEvent(const OS::KeyEvent& event);
     //@}
