@@ -35,6 +35,7 @@ namespace Lum {
   private:
     Lum::Object *main;
     Lum::Object *reference;
+    OS::FillRef background;
     bool        seamless;
     bool        border;
 
@@ -43,7 +44,7 @@ namespace Lum {
     ~Popup();
 
     void SetMain(Lum::Object* main, bool seamless=true, bool border=true);
-
+    void SetBackground(OS::Fill* background);
     void SetReference(Lum::Object* object);
     Lum::Object* GetReference() const;
 
