@@ -31,6 +31,9 @@ namespace Lum {
   {
   public:
     Radio();
+
+    static Radio* Create();
+    static Radio* Create(Base::Model* model, size_t index);
   };
 }
 
