@@ -86,21 +86,12 @@ namespace Lum {
     bool                  autoHSize;
     bool                  autoVSize;
 
-    size_t                currentPos;          //! The current shown position (top row)
-    size_t                scrolling;           //! The scrolling delta, if scrolling is active
-    bool                  scrollRefresh;
-    Model::ActionRef      scrollTimerAction;   //! Timer for scrolling updates
-    int                   px;                  //! x position of mouse on press
-    int                   py;                  //! y position of mouse on press
-    size_t                ppos;                //! top line on mouse press
-    int                   my;                  //! y position of mouse on last move
-    int                   sy;                  //! y position of mouse on the last timer event
-    Lum::Base::SystemTime mouseTimer;          //! Count time between mouse move events
-    Model::ActionRef      mouseTimerAction;    //! Timer for mouse press and hold detection
-
     size_t                tableWidth;
     size_t                rowHeight;
 
+    bool                  fullRefresh;
+    size_t                top;
+
     FormatProvider        *formatProvider;
     OS::FontRef           font;
 
