@@ -79,9 +79,9 @@ namespace Lum {
       Lum::Object* GetCurrentFocusObject() const;
       void SetFocusFirst();
       void SetFocusNext();
-      void SetFocusLast();
+      void SetFocusPrevious();
       void SetFocusNextInGroup();
-      void SetFocusLastInGroup();
+      void SetFocusPreviousInGroup();
       void Activate();
       void Deactivate();
       bool HandleEvent(OS::KeyEvent* event);
