@@ -490,7 +490,7 @@ namespace Lum {
 
     // Draw optional image
     if (ShowImage()) {
-      image->Draw(draw,imageX,imageY,image->GetWidth(),image->GetHeight());
+      image->Draw(draw,imageX,imageY);
     }
 
     draw->activated=false;
