@@ -19,12 +19,14 @@
 
 #include <Lum/DateSelect.h>
 
+#include <Lum/Dlg/ValuePopup.h>
+
 #include <Lum/DateView.h>
 
 namespace Lum {
 
   DateSelect::DateSelect()
-  : PopupControl(new DateView()),popup(NULL)
+  : PopupControl(new DateView())
   {
     if (!OS::display->GetTheme()->FocusOnEditableControlsOnly()) {
       SetCanFocus(true);
@@ -37,45 +39,16 @@ namespace Lum {
     // no code
   }
 
-  DateSelect::DateSelectPopup::DateSelectPopup(DateSelect *select)
-  : select(select),
-    picker(new DatePicker())
-  {
-    Observe(picker->GetSelectAction());
-  }
-
-  DateSelect::DateSelectPopup::~DateSelectPopup()
-  {
-    // no code
-  }
-
-  void DateSelect::DateSelectPopup::PreInit()
-  {
-    picker->SetFlex(true,true);
-    picker->SetModel(select->GetModel());
-
-    SetMain(picker);
-
-    Popup::PreInit();
-  }
-
-  void DateSelect::DateSelectPopup::Resync(Base::Model* model, const Base::ResyncMsg& msg)
-  {
-    if (model==picker->GetSelectAction() && picker->GetSelectAction()->IsFinished()) {
-      select->GetModel()->Save();
-      Exit();
-    }
-
-    Popup::Resync(model,msg);
-  }
-
   void DateSelect::OpenPopup()
   {
+    Dlg::DateValuePopup *popup;
+
     GetModel()->Push();
 
-    popup=new DateSelectPopup(this);
+    popup=new Dlg::DateValuePopup();
     popup->SetParent(GetWindow());
     popup->SetReference(this);
+    popup->SetModel(GetModel());
 
     if (popup->Open()) {
       popup->EventLoop();
