@@ -26,6 +26,7 @@
 #include <Lum/Button.h>
 #include <Lum/ButtonRow.h>
 #include <Lum/Panel.h>
+#include <Lum/Space.h>
 #include <Lum/Text.h>
 
 namespace Lum {
@@ -185,14 +186,17 @@ namespace Lum {
         SetMain(vPanel);
       }
       else if (OS::display->GetTheme()->GetDialogActionPosition()==OS::Theme::actionPosVertButtonRowRight) {
-        Button    *button;
-        Panel     *hPanel;
-        Panel     *vPanel;
+        Button *button;
+        Panel  *hPanel;
+        Panel  *vPanel;
+        bool   addSpace=false;
 
         hPanel=HPanel::Create(true,true);
 
         vPanel=VPanel::Create(false,true);
 
+        vPanel->Add(new VSpace(Space::sizeEmpty,true));
+
         // Optionals
         for (size_t i=0; i<actions.size(); i++) {
           if (actions[i].GetType()==ActionInfo::typeOptional) {
@@ -202,10 +206,13 @@ namespace Lum {
                                   Button::typeNormal,
                                   true,false);
             vPanel->Add(button);
+            addSpace=true;
           }
         }
 
-        vPanel->AddSpace(true);
+        if (addSpace) {
+          vPanel->AddSpace(true);
+        }
 
         // Additional
         for (size_t i=0; i<actions.size(); i++) {
