@@ -21,6 +21,7 @@
 
 #include <Lum/Base/Util.h>
 
+#include <Lum/OS/Behaviour.h>
 #include <Lum/OS/Display.h>
 
 namespace Lum {
@@ -129,7 +130,11 @@ namespace Lum {
 
   bool Knob::HandleMouseEvent(const OS::MouseEvent& event)
   {
-    if (!visible || !adjustment.Valid() || !adjustment->IsValid() || !adjustment->IsEnabled()) {
+    if (!visible ||
+        !adjustment.Valid() ||
+        !adjustment->IsValid() ||
+        !adjustment->IsEnabled() ||
+        !OS::display->GetBehaviour()->ScrollKnobActive()) {
       return false;
     }
 
@@ -189,6 +194,14 @@ namespace Lum {
 
   bool Knob::HandleKeyEvent(const OS::KeyEvent& event)
   {
+    if (!visible ||
+        !adjustment.Valid() ||
+        !adjustment->IsValid() ||
+        !adjustment->IsEnabled() ||
+        !OS::display->GetBehaviour()->ScrollKnobActive()) {
+      return false;
+    }
+
     if (event.type==OS::KeyEvent::down) {
       if ((vert && event.key==OS::keyUp) ||
           (!vert && event.key==OS::keyLeft)) {
