@@ -22,6 +22,8 @@
 #include <Lum/Base/Size.h>
 #include <Lum/Base/Util.h>
 
+#include <cmath>
+
 #define DEFAULT_PROP_FONT_SIZE 12
 #define DEFAULT_FIXED_FONT_SIZE 13
 
@@ -415,6 +417,11 @@ namespace Lum {
         draw->PopForeground();
       }
 
+      bool Theme::IsLightTheme() const
+      {
+        return true;
+      }
+
       size_t Theme::GetHorizontalFuelBarMinWidth() const
       {
         return 0;
@@ -435,6 +442,19 @@ namespace Lum {
         return 0;
       }
 
+      size_t Theme::GetCursorWidth(OS::Font* font) const
+      {
+        size_t result;
+
+        result=(size_t)std::ceil(font->pixelHeight*0.04);
+
+        if (result==0) {
+          result=1;
+        }
+
+        return result;
+      }
+
       int Theme::GetSubMenuHorizontalOffset() const
       {
         return 0;
@@ -559,7 +579,9 @@ namespace Lum {
         bool GetAlpha() const;
         unsigned long GetDrawCap() const;
 
-        void Draw(OS::DrawInfo* draw, int x, int y, size_t w, size_t h);
+        void Draw(OS::DrawInfo* draw, int x, int y);
+        void DrawScaled(OS::DrawInfo* draw, int x, int y, size_t w, size_t h);
+        void DrawStretched(OS::DrawInfo* draw, int x, int y, size_t w, size_t h);
       };
 
       /**
@@ -760,7 +782,21 @@ namespace Lum {
         draw->PopForeground();
       }
 
-      void InternalImage::Draw(OS::DrawInfo* draw, int x, int y, size_t width, size_t height)
+      void InternalImage::Draw(OS::DrawInfo* draw, int x, int y)
+      {
+        DrawScaled(draw,x,y,GetWidth(),GetHeight());
+      }
+
+      void InternalImage::DrawStretched(OS::DrawInfo* draw,
+                                        int x, int y,
+                                        size_t width, size_t height)
+      {
+        DrawScaled(draw,x,y,width,height);
+      }
+
+      void InternalImage::DrawScaled(OS::DrawInfo* draw,
+                                     int x, int y,
+                                     size_t width, size_t height)
       {
         OS::DrawInfo::Point points[3];
         OS::FontRef         font=OS::display->GetFont();
