@@ -22,6 +22,7 @@
 #include <hildon/hildon-version.h>
 
 #include <Lum/Array.h>
+#include <Lum/BooleanButton.h>
 #include <Lum/Button.h>
 #include <Lum/Dialog.h>
 #include <Lum/Menu.h>
@@ -33,7 +34,6 @@
 #include <Lum/OS/X11/Window.h>
 #include <Lum/OS/X11/GtkTheme.h>
 
-#include <iostream>
 namespace Lum {
   namespace OS {
     namespace Hildon {
@@ -122,15 +122,14 @@ namespace Lum {
           void Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
           {
             if (model==GetClosedAction() && GetClosedAction()->IsFinished()) {
-              Close();
+              Hide();
+              return;
             }
             else if (dynamic_cast<Model::Action*>(model)!=NULL) {
               Model::ActionRef action=dynamic_cast<Lum::Model::Action*>(model);
 
               if (actions.find(action)!=actions.end()) {
                 if (action->IsEnabled() && action->IsFinished() && IsOpen()) {
-                  // If the action was triggered while enabled and by us (IsOpen()==true)
-                  // hide ourself.
                   Hide();
                   return;
                 }
@@ -142,8 +141,8 @@ namespace Lum {
         };
 
       private:
-        Menu  *menu;
-        Popup *popup;
+        Menu      *menu;
+        MenuPopup *popup;
 
       private:
         void CountModels(Def::Menu* def,
@@ -229,6 +228,9 @@ namespace Lum {
           delete menu;
           delete popup;
 
+          menu=NULL;
+          popup=NULL;
+
           if (CanBeDisplayedAsButtonMenuPopup(def,excludes)) {
             popup=new MenuPopup(def,excludes);
             popup->SetParent(window);
@@ -366,8 +368,7 @@ namespace Lum {
             Observe(action);
 
             button=Button::Create(view->GetDesc().GetLabel(),action,true,true);
-            button->SetMaxHeight(Lum::Base::Size::pixel,
-                                 2*display->GetTheme()->GetFingerFriendlyMinHeight().GetSize());
+            button->SetMaxHeight(Lum::Base::Size::pixel,105); //Shouldn't be hardcoded
 
             Add(button);
           }
@@ -454,6 +455,24 @@ namespace Lum {
       };
 #endif
 
+      bool Behaviour::FocusObjectOnWindowFocusIn() const
+      {
+#if HILDON_CHECK_VERSION(2,1,0)
+        return true;
+#else
+        return false;
+#endif
+      }
+
+      bool Behaviour::ScrollKnobActive() const
+      {
+#if HILDON_CHECK_VERSION(2,1,0)
+        return false;
+#else
+        return true;
+#endif
+      }
+
       void Behaviour::ApplyMultiViewDlg(Dialog* dialog,
                                         const Def::MultiView& multiView) const
       {
@@ -511,13 +530,27 @@ namespace Lum {
         TableIndexValueButton *control;
 
         control=new TableIndexValueButton(def.GetChoices());
-        control->SetFlex(true,false);
         control->SetModel(def.GetValue());
         control->SetLabel(def.GetDesc().GetLabel());
 
         return control;
 #else
         return DefaultBehaviour::GetOneOfManyControl(def);
+#endif
+      }
+
+      Lum::Object* Behaviour::GetBooleanControl(Def::Boolean& def) const
+      {
+#if HILDON_CHECK_VERSION(2,1,0)
+        BooleanButton *control;
+
+        control=new BooleanButton();
+        control->SetModel(def.GetBoolean());
+        control->SetLabel(def.GetDesc().GetLabel());
+
+        return control;
+#else
+        return DefaultBehaviour::GetBooleanControl(def);
 #endif
       }
     }
