@@ -21,7 +21,7 @@
 
 #include <X11/Xatom.h>
 
-#include <hildon-input-method/hildon-im-protocol.h>
+#include <hildon/hildon-version.h>
 #include <gtk/gtkenums.h>
 
 #include <Lum/Base/String.h>
@@ -42,6 +42,11 @@ namespace Lum {
       : window(NULL),
         object(NULL)
       {
+#if HILDON_CHECK_VERSION(2,1,0)
+        triggerType=HILDON_IM_TRIGGER_FINGER;
+#else
+        triggerType=HILDON_IM_TRIGGER_STYLUS;
+#endif
         // no code
       }
 
@@ -70,10 +75,22 @@ namespace Lum {
                                           Lum::Object* oldObject,
                                           Lum::Object* newObject)
       {
+        //std::cout << "Focus change: " << window << " " << oldObject << " " << newObject << std::endl;
         if (this->window!=window) {
+          //std::cerr << "Wrong window!" << std::endl;
           return;
         }
 
+#if HILDON_CHECK_VERSION(2,1,0)
+        if (newObject!=NULL &&
+                 newObject->RequestsKeyboard() &&
+                 newObject->GetIMHandler()!=NULL) {
+          object=newObject;
+        }
+        else {
+          object=NULL;
+        }
+#else
         if (object==newObject) {
           return;
         }
@@ -90,6 +107,27 @@ namespace Lum {
           object=NULL;
           HideKeyboard();
         }
+#endif
+      }
+
+      void KeyboardManager::OnMouseClick(Window* window,
+                                         Lum::Object* object,
+                                         MouseEvent* event)
+      {
+        if (this->window!=window) {
+          //std::cerr << "Wrong window!" << std::endl;
+          return;
+        }
+
+#if HILDON_CHECK_VERSION(2,1,0)
+        if (object!=NULL &&
+            this->object==object &&
+            object->RequestsKeyboard() &&
+            object->GetIMHandler()!=NULL &&
+            object->GetIMHandler()->IsInEditingArea(event)) {
+          ShowKeyboard();
+        }
+#endif
       }
 
       bool KeyboardManager::HandleEvent(OS::Window* window, OS::Event* event)
@@ -118,63 +156,24 @@ namespace Lum {
           return false;
         }
 
-        if (x11Event.xclient.message_type==XInternAtom(display->display,HILDON_IM_COM_NAME,false)) {
+        if (x11Event.xclient.message_type==XInternAtom(display->display,
+                                                       HILDON_IM_COM_NAME,false)) {
           HildonIMComMessage *msg = (HildonIMComMessage *)&x11Event.xclient.data;
 
           if (msg->type==HILDON_IM_CONTEXT_HANDLE_ENTER) {
-            X11::KeyEvent keyEvent;
-
-            keyEvent.key=keyReturn;
-            keyEvent.qualifier=0;
-            keyEvent.type=KeyEvent::down;
-
-            window->HandleEvent(&keyEvent);
-
-            keyEvent.type=KeyEvent::up;
-
-            window->HandleEvent(&keyEvent);
+            SendKey(keyReturn);
             return true;
           }
           else if (msg->type==HILDON_IM_CONTEXT_HANDLE_TAB) {
-            X11::KeyEvent keyEvent;
-
-            keyEvent.key=keyTab;
-            keyEvent.qualifier=0;
-            keyEvent.type=KeyEvent::down;
-
-            window->HandleEvent(&keyEvent);
-
-            keyEvent.type=KeyEvent::up;
-
-            window->HandleEvent(&keyEvent);
+            SendKey(keyTab);
             return true;
           }
           else if (msg->type==HILDON_IM_CONTEXT_HANDLE_BACKSPACE) {
-            X11::KeyEvent keyEvent;
-
-            keyEvent.key=keyBackspace;
-            keyEvent.qualifier=0;
-            keyEvent.type=KeyEvent::down;
-
-            window->HandleEvent(&keyEvent);
-
-            keyEvent.type=KeyEvent::up;
-
-            window->HandleEvent(&keyEvent);
+            SendKey(keyBackspace);
             return true;
           }
           else if (msg->type==HILDON_IM_CONTEXT_HANDLE_SPACE) {
-            X11::KeyEvent keyEvent;
-
-            keyEvent.key=keySpace;
-            keyEvent.qualifier=0;
-            keyEvent.type=KeyEvent::down;
-
-            window->HandleEvent(&keyEvent);
-
-            keyEvent.type=KeyEvent::up;
-
-            window->HandleEvent(&keyEvent);
+            SendKey(keySpace);
             return true;
           }
           else if (msg->type==HILDON_IM_CONTEXT_CONFIRM_SENTENCE_START) {
@@ -185,20 +184,28 @@ namespace Lum {
             memset(&event,0,sizeof(event));
             event.xclient.type=ClientMessage;
             event.xclient.window=imWindow;
-            event.xclient.message_type=XInternAtom(display->display,HILDON_IM_ACTIVATE_NAME,False);
-            event.xclient.format=8;
-            event.xclient.data.l[0]=dynamic_cast<X11::Window*>(window)->GetDrawable();
-            event.xclient.data.l[1]=dynamic_cast<X11::Window*>(window)->GetDrawable();
-            event.xclient.data.l[2]=0x03;
-            event.xclient.data.l[3]=0x00000007;
-            event.xclient.data.l[4]=0;
+            event.xclient.message_type=XInternAtom(display->display,
+                                                   HILDON_IM_ACTIVATE_NAME,false);
+            event.xclient.format=HILDON_IM_ACTIVATE_FORMAT;
+
+            HildonIMActivateMessage *msg = reinterpret_cast<HildonIMActivateMessage *>(&event.xclient.data);
+
+            msg->input_window=dynamic_cast<X11::Window*>(window)->GetDrawable();
+            msg->app_window=dynamic_cast<X11::Window*>(window)->GetDrawable();
+            msg->cmd=HILDON_IM_UPP;
+            msg->trigger=triggerType;
+
             if (XSendEvent(display->display,imWindow,False,NoEventMask,&event)==0) {
               std::cerr << "Error while sending _HILDON_IM_ACTIVATE" << std::endl;
             }
+            XSync(display->display,False);
           }
           else if (msg->type==HILDON_IM_CONTEXT_FLUSH_PREEDIT) {
             //std::cout << "HILDON_IM_CONTEXT_FLUSH_PREEDIT" << std::endl;
           }
+          /*else if (msg->type==HILDON_IM_CONTEXT_CANCEL_PREEDIT) {
+            //std::cout << "HILDON_IM_CONTEXT_CANCEL_PREEDIT" << std::endl;
+          }*/
           else if (msg->type==HILDON_IM_CONTEXT_BUFFERED_MODE) {
             //std::cout << "HILDON_IM_CONTEXT_BUFFERED_MODE" << std::endl;
           }
@@ -211,6 +218,9 @@ namespace Lum {
           else if (msg->type==HILDON_IM_CONTEXT_SURROUNDING_MODE) {
             //std::cout << "HILDON_IM_CONTEXT_SURROUNDING_MODE" << std::endl;
           }
+          /*else if (msg->type==HILDON_IM_CONTEXT_PREEDIT_MODE) {
+            //std::cout << "HILDON_IM_CONTEXT_PREEEDIT_MODE" << std::endl;
+          }*/
           else if (msg->type==HILDON_IM_CONTEXT_CLIPBOARD_COPY) {
             //std::cout << "HILDON_IM_CONTEXT_CLIPBOARD_COPY" << std::endl;
             Copy();
@@ -225,30 +235,18 @@ namespace Lum {
           }
           else if (msg->type==HILDON_IM_CONTEXT_CLIPBOARD_SELECTION_QUERY) {
             //std::cout << "HILDON_IM_CONTEXT_CLIPBOARD_SELECTION_QUERY" << std::endl;
-
-            ::XEvent                event;
-            bool                    hasSelection;
-
-            hasSelection=object!=NULL && object->CanExecuteAction(Lum::Base::Datatype(Lum::Base::Datatype::text,
-                                                                                      Lum::Base::Datatype::plain),
-                                                                  Lum::Base::DataExchangeObject::actionCopy);
-
-            std::cout << "HasSelection: " << hasSelection << std::endl;
-
-            memset(&event,0,sizeof(event));
-            event.xclient.type=ClientMessage;
-            event.xclient.window=imWindow;
-            event.xclient.message_type=XInternAtom(display->display,HILDON_IM_CLIPBOARD_SELECTION_REPLY_NAME,False);
-            event.xclient.format=HILDON_IM_CLIPBOARD_SELECTION_REPLY_FORMAT;
-            event.xclient.data.l[0]=hasSelection;
-
-            if (XSendEvent(display->display,imWindow,False,NoEventMask,&event)==0) {
-              std::cerr << "Error while sending HILDON_IM_CLIPBOARD_SELECTION_REPLY" << std::endl;
-            }
+            SendClipboardSelectionQueryResponse();
           }
           else if (msg->type==HILDON_IM_CONTEXT_REQUEST_SURROUNDING) {
             //std::cout << "HILDON_IM_CONTEXT_REQUEST_SURROUNDING" << std::endl;
           }
+#if HILDON_CHECK_VERSION(2,1,0)
+          else if (msg->type==HILDON_IM_CONTEXT_REQUEST_SURROUNDING_FULL) {
+            //std::cout << "HILDON_IM_CONTEXT_REQUEST_SURROUNDING_FULL" << std::endl;
+
+            SendSurrounding();
+          }
+#endif
           else if (msg->type==HILDON_IM_CONTEXT_WIDGET_CHANGED) {
             //std::cout << "HILDON_IM_CONTEXT_WIDGET_CHANGED" << std::endl;
           }
@@ -258,14 +256,23 @@ namespace Lum {
           else if (msg->type==HILDON_IM_CONTEXT_CLEAR_STICKY) {
             //std::cout << "HILDON_IM_CONTEXT_CLEAR_STICKY" << std::endl;
           }
-          //else if (msg->type==HILDON_IM_CONTEXT_ENTER_ON_FOCUS) {
+          else if (msg->type==HILDON_IM_CONTEXT_ENTER_ON_FOCUS) {
             //std::cout << "HILDON_IM_CONTEXT_ENTER_ON_FOCUS" << std::endl;
-          //}
+          }
+          /*else if (msg->type==HILDON_IM_CONTEXT_SPACE_AFTER_COMMIT) {
+            //std::cout << "HILDON_IM_CONTEXT_SPACE_AFTER_COMMIT" << std::endl;
+          }
+          else if (msg->type==HILDON_IM_CONTEXT_NO_SPACE_AFTER_COMMIT) {
+            //std::cout << "HILDON_IM_CONTEXT_NO_SPACE_AFTER_COMMIT" << std::endl;
+          }*/
           else {
-            std::cout << "got HILDON-COM " << std::hex << msg->type << std::endl;
+            //std::cout << "got HILDON-COM " << std::hex << msg->type << std::endl;
           }
         }
-        else if (x11Event.xclient.message_type==XInternAtom(display->display,HILDON_IM_INSERT_UTF8_NAME,false)) {
+        else if (x11Event.xclient.message_type==XInternAtom(display->display,
+                                                            HILDON_IM_INSERT_UTF8_NAME,
+                                                            false)) {
+          //std::cout << "HILDON_IM_INSERT_UTF8" << std::endl;
           X11::KeyEvent keyEvent;
           std::string   inputText;
           size_t        i;
@@ -289,16 +296,204 @@ namespace Lum {
 
           window->HandleEvent(&keyEvent);
         }
-        else if (x11Event.xclient.message_type==XInternAtom(display->display,HILDON_IM_SURROUNDING_CONTENT_NAME,false)) {
-          //std::cout << HILDON_IM_SURROUNDING_CONTENT_NAME << std::endl;
+        else if (x11Event.xclient.message_type==XInternAtom(display->display,
+                                                            HILDON_IM_SURROUNDING_CONTENT_NAME,
+                                                            false)) {
+          HildonIMSurroundingContentMessage *msg = reinterpret_cast<HildonIMSurroundingContentMessage *>(&x11Event.xclient.data);
+
+          //std::cout << "HILDON_IM_SURROUNDING_CONTENT '" << msg->surrounding << "'" << std::endl;
+
+          if (msg->msg_flag==HILDON_IM_MSG_START) {
+            buffer.clear();
+          }
+
+          buffer+=msg->surrounding;
+
+
+          if (msg->msg_flag==HILDON_IM_MSG_END) {
+            assert(object!=NULL);
+            assert(object->GetIMHandler()!=NULL);
+            object->GetIMHandler()->SetData(Lum::Base::UTF8ToWString(buffer));
+          }
         }
-        else if (x11Event.xclient.message_type==XInternAtom(display->display,HILDON_IM_SURROUNDING_NAME,false)) {
-          //std::cout << HILDON_IM_SURROUNDING_NAME << std::endl;
+        else if (x11Event.xclient.message_type==XInternAtom(display->display,
+                                                            HILDON_IM_SURROUNDING_NAME,
+                                                            false)) {
+
+          HildonIMSurroundingMessage *msg = reinterpret_cast<HildonIMSurroundingMessage *>(&x11Event.xclient.data);
+
+          //std::cout << "HILDON_IM_SURROUNDING " << msg->commit_mode << " "  << msg->offset_is_relative << " " << msg->cursor_offset << std::endl;
+
+          assert(object!=NULL);
+          assert(object->GetIMHandler()!=NULL);
+
+          object->GetIMHandler()->SetCursorOffset(msg->cursor_offset);
         }
+        /*else if (x11Event.xclient.message_type==XInternAtom(display->display,
+                                                            HILDON_IM_INPUT_MODE_NAME,
+                                                            false)) {
+          //std::cout << "HILDON_IM_INPUT_MODE" << std::endl;
+        }*/
+        else if (x11Event.xclient.message_type==XInternAtom(display->display,
+                                                            HILDON_IM_KEY_EVENT_NAME,
+                                                            false)) {
+          //std::cout << "HILDON_IM_KEY_EVENT" << std::endl;
+        }
+        /*else if (x11Event.xclient.message_type==XInternAtom(display->display,
+                                                            HILDON_IM_PREEDIT_COMMITTED_NAME,
+                                                            false)) {
+          //std::cout << "HILDON_IM_PREEDIT_COMMITTED" << std::endl;
+        }
+        else if (x11Event.xclient.message_type==XInternAtom(display->display,
+                                                            HILDON_IM_PREEDIT_COMMITTED_CONTENT_NAME,
+                                                            false)) {
+          //std::cout << "HILDON_IM_PREEDIT_COMMITTED_CONTENT" << std::endl;
+        }*/
 
         return false;
       }
 
+      void KeyboardManager::SendSurrounding()
+      {
+        ::XEvent     event;
+        std::wstring data;
+        std::string  utf8Data;
+        size_t       cursorPos;
+        size_t       currentPos;
+        size_t       length;
+        ::Window     imWindow=GetKeyboardWindow();
+        X11::Display *display=dynamic_cast<X11::Display*>(Lum::OS::display);
+
+        if (display==NULL) {
+          return;
+        }
+
+        if (imWindow==None) {
+          return;
+        }
+
+        assert(object!=NULL);
+        assert(object->GetIMHandler()!=NULL);
+
+        object->GetIMHandler()->GetData(data,cursorPos);
+
+        utf8Data=Lum::Base::WStringToUTF8(data);
+
+        memset(&event,0,sizeof(event));
+        event.xclient.type=ClientMessage;
+        event.xclient.window=imWindow;
+        event.xclient.message_type=XInternAtom(display->display,
+                                               HILDON_IM_SURROUNDING_CONTENT_NAME,
+                                               false);
+        event.xclient.format=HILDON_IM_SURROUNDING_CONTENT_FORMAT;
+
+        HildonIMSurroundingContentMessage *msg = reinterpret_cast<HildonIMSurroundingContentMessage *>(&event.xclient.data);
+
+        currentPos=0;
+        length=std::min(utf8Data.length(),
+                        HILDON_IM_CLIENT_MESSAGE_BUFFER_SIZE);
+
+        msg->msg_flag=HILDON_IM_MSG_START;
+        strcpy(msg->surrounding,utf8Data.substr(currentPos,length).c_str());
+
+        if (XSendEvent(display->display,imWindow,False,NoEventMask,&event)==0) {
+          std::cerr << "Error while sending HILDON_IM_SURROUNDING_CONTENT" << std::endl;
+        }
+
+        if (utf8Data.length()>0) {
+          currentPos+=length;
+
+          while (currentPos<utf8Data.length()-1) {
+          length=std::min(utf8Data.length()-currentPos,
+                          HILDON_IM_CLIENT_MESSAGE_BUFFER_SIZE);
+
+            msg->msg_flag=HILDON_IM_MSG_CONTINUE;
+            strcpy(msg->surrounding,utf8Data.substr(currentPos,length).c_str());
+
+            if (XSendEvent(display->display,imWindow,False,NoEventMask,&event)==0) {
+              std::cerr << "Error while sending HILDON_IM_SURROUNDING_CONTENT" << std::endl;
+            }
+
+            currentPos+=length;
+          }
+        }
+
+        /*
+        msg->msg_flag=HILDON_IM_MSG_END;
+        msg->surrounding[0]='\0';
+
+        if (XSendEvent(display->display,imWindow,False,NoEventMask,&event)==0) {
+          std::cerr << "Error while sending HILDON_IM_SURROUNDING_CONTENT_NAME" << std::endl;
+        }*/
+
+        memset(&event,0,sizeof(event));
+        event.xclient.type=ClientMessage;
+        event.xclient.window=imWindow;
+        event.xclient.message_type=XInternAtom(display->display,
+                                               HILDON_IM_SURROUNDING_NAME,
+                                               false);
+        event.xclient.format=HILDON_IM_SURROUNDING_FORMAT;
+
+        HildonIMSurroundingMessage *smsg = reinterpret_cast<HildonIMSurroundingMessage *>(&event.xclient.data);
+
+        smsg->commit_mode=HILDON_IM_COMMIT_SURROUNDING;
+        smsg->offset_is_relative=0;
+        smsg->cursor_offset=cursorPos;
+
+        if (XSendEvent(display->display,imWindow,False,NoEventMask,&event)==0) {
+          std::cerr << "Error while sending HILDON_IM_SURROUNDING" << std::endl;
+        }
+      }
+
+      void KeyboardManager::SendClipboardSelectionQueryResponse()
+      {
+        ::XEvent     event;
+        bool         hasSelection;
+        ::Window     imWindow=GetKeyboardWindow();
+        X11::Display *display=dynamic_cast<X11::Display*>(Lum::OS::display);
+
+        if (display==NULL) {
+          return;
+        }
+
+        if (imWindow==None) {
+          return;
+        }
+
+        hasSelection=object!=NULL &&
+                     object->CanExecuteAction(Lum::Base::Datatype(Lum::Base::Datatype::text,
+                                                                  Lum::Base::Datatype::plain),
+                                              Lum::Base::DataExchangeObject::actionCopy);
+
+        memset(&event,0,sizeof(event));
+        event.xclient.type=ClientMessage;
+        event.xclient.window=imWindow;
+        event.xclient.message_type=XInternAtom(display->display,
+                                               HILDON_IM_CLIPBOARD_SELECTION_REPLY_NAME,
+                                               false);
+        event.xclient.format=HILDON_IM_CLIPBOARD_SELECTION_REPLY_FORMAT;
+        event.xclient.data.l[0]=hasSelection;
+
+        if (XSendEvent(display->display,imWindow,False,NoEventMask,&event)==0) {
+          std::cerr << "Error while sending HILDON_IM_CLIPBOARD_SELECTION_REPLY" << std::endl;
+        }
+      }
+
+      void KeyboardManager::SendKey(Key key)
+      {
+        X11::KeyEvent keyEvent;
+
+        keyEvent.key=key;
+        keyEvent.qualifier=0;
+        keyEvent.type=KeyEvent::down;
+
+        window->HandleEvent(&keyEvent);
+
+        keyEvent.type=KeyEvent::up;
+
+        window->HandleEvent(&keyEvent);
+      }
+
       void KeyboardManager::ShowKeyboard()
       {
         assert(window!=0);
@@ -306,12 +501,14 @@ namespace Lum {
         X11::Display *display=dynamic_cast<X11::Display*>(Lum::OS::display);
 
         if (display==NULL) {
+          std::cerr << "No display!" << std::endl;
           return;
         }
 
         ::Window imWindow=GetKeyboardWindow();
 
         if (imWindow==None) {
+          std::cerr << "No IM window!" << std::endl;
           return;
         }
 
@@ -320,7 +517,9 @@ namespace Lum {
         memset(&event,0,sizeof(event));
         event.xclient.type=ClientMessage;
         event.xclient.window=imWindow;
-        event.xclient.message_type=XInternAtom(display->display,HILDON_IM_ACTIVATE_NAME,False);
+        event.xclient.message_type=XInternAtom(display->display,
+                                               HILDON_IM_ACTIVATE_NAME,
+                                               false);
         event.xclient.format=HILDON_IM_ACTIVATE_FORMAT;
 
         HildonIMActivateMessage *msg = reinterpret_cast<HildonIMActivateMessage *>(&event.xclient.data);
@@ -328,8 +527,7 @@ namespace Lum {
         msg->input_window=dynamic_cast<X11::Window*>(window)->GetDrawable();
         msg->app_window=dynamic_cast<X11::Window*>(window)->GetDrawable();
         msg->cmd=HILDON_IM_SETNSHOW;
-        //msg->input_mode=HILDON_GTK_INPUT_MODE_FULL;
-        msg->trigger=HILDON_IM_TRIGGER_STYLUS;
+        msg->trigger=triggerType;
 
         if (XSendEvent(display->display,imWindow,False,NoEventMask,&event)==0) {
           std::cerr << "Error while sending OpenKeyboard (_HILDON_IM_ACTIVATE)" << std::endl;
@@ -357,7 +555,8 @@ namespace Lum {
         memset(&event,0,sizeof(event));
         event.xclient.type=ClientMessage;
         event.xclient.window=imWindow;
-        event.xclient.message_type=XInternAtom(display->display,HILDON_IM_ACTIVATE_NAME,False);
+        event.xclient.message_type=XInternAtom(display->display,HILDON_IM_ACTIVATE_NAME,
+                                               false);
         event.xclient.format=HILDON_IM_ACTIVATE_FORMAT;
 
         HildonIMActivateMessage *msg = reinterpret_cast<HildonIMActivateMessage *>(&event.xclient.data);
@@ -366,7 +565,7 @@ namespace Lum {
         msg->app_window=0;
         msg->cmd=HILDON_IM_HIDE;
         //msg->input_mode=HILDON_GTK_INPUT_MODE_FULL;
-        msg->trigger=HILDON_IM_TRIGGER_STYLUS;
+        msg->trigger=triggerType;
 
         if (XSendEvent(display->display,imWindow,False,NoEventMask,&event)==0) {
           std::cerr << "Error while sending 'CloseKeyboard' (_HILDON_IM_ACTIVATE)" << std::endl;
