@@ -1279,7 +1279,8 @@ namespace Lum {
 
         win=winList.begin();
         while (win!=winList.end()) {
-          std::cerr << "Warning: window '" <<Lum::Base::WStringToString((*win)->GetTitle()) << "' not explicitely closed!" << std::endl;
+          std::cerr << "Warning: window " << *win << " not explicitely closed!" << std::endl;
+          std::cerr << "  Window title: '" <<Lum::Base::WStringToString((*win)->GetTitle()) << "'" << std::endl;
           (*win)->Close(); // removes itself from list
 
           win=winList.begin();
@@ -1748,25 +1749,25 @@ namespace Lum {
         currentTray=GetTray(e.xany.window);
 
         if (currentWin!=NULL) {
-          OS::Event *event;
+          OS::EventRef event;
 
           event=X11::GetEvent(e,currentWin->xic);
 
-          if (event!=NULL) {
+          if (event.Valid()) {
             do {
               event->reUse=false;
-              currentWin->HandleEvent(event);
+              currentWin->HandleEvent(event.Get());
             }
             while (event->reUse);
           }
         }
         else if (currentTray!=NULL) {
-          OS::Event *event;
+          OS::EventRef event;
 
           event=X11::GetEvent(e,currentTray->xic);
 
-          if (event!=NULL) {
-            currentTray->HandleEvent(event);
+          if (event.Valid()) {
+            currentTray->HandleEvent(event.Get());
           }
         }
       }
