@@ -231,7 +231,7 @@ namespace Lum {
     assert(image.Valid());
 
     image->Draw(GetDrawInfo(),
-                this->x,this->y,this->width,this->height);
+                this->x,this->y);
   }
 
 }
