@@ -23,6 +23,7 @@ namespace Lum {
 
   Table::Table()
   : object(NULL),hScroller(NULL),vScroller(NULL),
+    kineticScroller(new KineticScroller()),
     hVisible(false),vVisible(false),
     table(new TableView()),
     header(NULL),headerModel(new Model::HeaderImpl()),showHeader(false),
@@ -39,6 +40,7 @@ namespace Lum {
 
   Table::~Table()
   {
+    delete kineticScroller;
     delete object;
     delete hScroller;
     delete vScroller;
@@ -189,6 +191,8 @@ namespace Lum {
     minHeight=std::max(vScroller->GetOHeight(),minHeight);
     height=std::max(vScroller->GetOHeight(),height);
 
+    kineticScroller->SetObjects(this,table);
+
     Group::CalcSize();
   }
 
@@ -258,46 +262,60 @@ namespace Lum {
     return vScroller->GetOWidth();
   }
 
+  bool Table::InterceptMouseEvents() const
+  {
+    return true;
+  }
+
+  bool Table::HandleMouseEvent(const OS::MouseEvent& event)
+  {
+    if (table==NULL) {
+      return false;
+    }
+
+    if (kineticScroller->HandleMouseEvent(event)) {
+      return true;
+    }
+
+    return Group::HandleMouseEvent(event);
+  }
+
   void Table::Resync(Base::Model* model, const Base::ResyncMsg& msg)
   {
-    if (table!=NULL) {
-      if (model==table->GetHAdjustment() && table->GetHAdjustment()->IsValid()) {
-        if (visible) {
-          OS::display->QueueActionForEventLoop(refreshAction);
-        }
-        else {
-          SetRelayout();
-        }
-        return;
+    if (object==NULL) {
+      return;
+    }
+
+    if (model==table->GetHAdjustment() ||
+        model==table->GetVAdjustment()) {
+      if (visible) {
+        OS::display->QueueActionForEventLoop(refreshAction);
       }
-      else if (model==table->GetVAdjustment()  && table->GetVAdjustment()->IsValid()) {
-        if (visible) {
-          OS::display->QueueActionForEventLoop(refreshAction);
-        }
-        else {
-          SetRelayout();
-        }
-        return;
+      else {
+        SetRelayout();
       }
-      else if (model==refreshAction && refreshAction->IsFinished()) {
-        if ((hVisible &&
-             (!table->GetHAdjustment()->IsValid() ||
-              table->GetHAdjustment()->GetVisible()==table->GetHAdjustment()->GetTotal())) ||
-            (!hVisible &&
-             table->GetHAdjustment()->IsValid() &&
-             table->GetHAdjustment()->GetVisible()<table->GetHAdjustment()->GetTotal()) ||
-            (vVisible &&
-             (!table->GetVAdjustment()->IsValid() ||
-              table->GetVAdjustment()->GetVisible()==table->GetVAdjustment()->GetTotal())) ||
-            (!vVisible &&
-             table->GetVAdjustment()->IsValid() &&
-             table->GetVAdjustment()->GetVisible()<table->GetVAdjustment()->GetTotal())) {
-          SetRelayout();
-          Redraw();
-        }
+      return;
+    }
+    else if (model==refreshAction && refreshAction->IsFinished()) {
+      if ((hVisible &&
+           (!table->GetHAdjustment()->IsValid() ||
+            table->GetHAdjustment()->GetVisible()==table->GetHAdjustment()->GetTotal())) ||
+          (!hVisible &&
+           table->GetHAdjustment()->IsValid() &&
+           table->GetHAdjustment()->GetVisible()<table->GetHAdjustment()->GetTotal()) ||
+          (vVisible &&
+           (!table->GetVAdjustment()->IsValid() ||
+            table->GetVAdjustment()->GetVisible()==table->GetVAdjustment()->GetTotal())) ||
+          (!vVisible &&
+           table->GetVAdjustment()->IsValid() &&
+           table->GetVAdjustment()->GetVisible()<table->GetVAdjustment()->GetTotal())) {
+        SetRelayout();
+        Redraw();
       }
     }
 
+    kineticScroller->Resync(model,msg);
+
     Group::Resync(model,msg);
   }
 }
