@@ -29,8 +29,6 @@
 
 namespace Lum {
 
-#define STEP 10
-
   static ControllerRef controller(new TableView::Controller);
 
   TableView::FormatProvider::~FormatProvider()
@@ -64,18 +62,10 @@ namespace Lum {
     autoFitColumns(false),
     autoHSize(false),
     autoVSize(false),
-    currentPos(1),
-    scrolling(false),
-    scrollRefresh(false),
-    scrollTimerAction(new Model::Action()),
-    px(0),
-    py(0),
-    ppos(0),
-    my(0),
-    sy(0),
-    mouseTimerAction(new Model::Action()),
     tableWidth(0),
     rowHeight(0),
+    fullRefresh(true),
+    top(0),
     formatProvider(new FormatProvider()),
     font(OS::display->GetFont())
   {
@@ -87,9 +77,6 @@ namespace Lum {
     RequestFocus();
 
     SetController(Lum::controller);
-
-    Observe(scrollTimerAction);
-    Observe(mouseTimerAction);
   }
 
   TableView::~TableView()
@@ -113,7 +100,7 @@ namespace Lum {
 
     size_t start;
 
-    y=((my-this->y)+currentPos-1+rowHeight-1)/rowHeight;
+    y=((my-this->y)+vAdjustment->GetTop()-1+rowHeight-1)/rowHeight;
 
     if (y==0 || y>model->GetRows()) {
       return false;
@@ -211,7 +198,7 @@ namespace Lum {
     this->model=dynamic_cast<Model::Table*>(model);
 
     if (this->model.Valid())  {
-      currentPos=1;
+      vAdjustment->SetTop(1);
     }
 
     Control::SetModel(this->model);
@@ -424,25 +411,25 @@ namespace Lum {
 
     // Is line visible at all?
 
-    if (end+1<currentPos) {
+    if (end+1<vAdjustment->GetTop()) {
       return false;
     }
 
-    if (start+1>currentPos+height-1) {
+    if (start+1>vAdjustment->GetTop()+height-1) {
       return false;
     }
 
     // From now on at least something must be visible
 
-    if (start<currentPos-1) {
-      start=currentPos-1;
+    if (start<vAdjustment->GetTop()-1) {
+      start=vAdjustment->GetTop()-1;
     }
 
-    if (end>currentPos-1+height-1) {
-      end=currentPos-1+height-1;
+    if (end>vAdjustment->GetTop()-1+height-1) {
+      end=vAdjustment->GetTop()-1+height-1;
     }
 
-    ly=start+1-currentPos+y;
+    ly=start+1-vAdjustment->GetTop()+y;
     lheight=end-start+1;
 
     return true;
@@ -594,7 +581,10 @@ namespace Lum {
       back->Draw(draw,xPos,yPos,width,height,xPos,yPos,width,height);
     }
 
-    if (selected && selection.Valid() && !selection->IsLineSelected(y) && HasFocus()) {
+    if (selected &&
+        selection.Valid() &&
+        !selection->IsLineSelected(y) &&
+        HasFocus()) {
       OS::FrameRef focus=OS::display->GetFrame(OS::Display::focusFrameIndex);
 
       /* Draw a frame arond one cell */
@@ -660,7 +650,9 @@ namespace Lum {
 
     }
 
-    if (selection.Valid() && selection->IsLineSelected(line) && HasFocus()) {
+    if (selection.Valid() &&
+        selection->IsLineSelected(line) &&
+        HasFocus()) {
       OS::FrameRef focus=OS::display->GetFrame(OS::Display::focusFrameIndex);
 
       /* Draw a frame arond whole line */
@@ -684,11 +676,11 @@ namespace Lum {
     draw->PushFont(font);
     draw->focused=HasFocus();
 
-    startLine=(currentPos-1+start)/rowHeight;
-    endLine=(currentPos-1+start+height-1)/rowHeight;
+    startLine=(vAdjustment->GetTop()-1+start)/rowHeight;
+    endLine=(vAdjustment->GetTop()-1+start+height-1)/rowHeight;
 
-    startPos=startLine*rowHeight+1-currentPos;
-    endPos=(endLine+1)*rowHeight+1-currentPos-1;
+    startPos=startLine*rowHeight+1-vAdjustment->GetTop();
+    endPos=(endLine+1)*rowHeight+1-vAdjustment->GetTop()-1;
 
     startPos+=y;
     endPos+=y;
@@ -715,7 +707,6 @@ namespace Lum {
     size_t lh=0;
 
     if (GetLineBounds(line,ly,lh)) {
-      scrollRefresh=false;
       Redraw(this->x,ly,width,lh);
     }
   }
@@ -774,9 +765,6 @@ namespace Lum {
 
   void TableView::Layout()
   {
-    // If we are still scrolling during a resize event, stop scrolling
-    scrollRefresh=false;
-
     CalculateRowHeight();
 
     if (model.Valid() && header.Valid() && autoFitColumns) {
@@ -827,77 +815,36 @@ namespace Lum {
 
     RecalcTableWidth();
 
-    scrolling=(currentPos!=vAdjustment->GetTop());
-
     if (OS::display->GetType()==OS::Display::typeTextual) {
-      currentPos=vAdjustment->GetTop();
       DrawFull(draw);
       return;
     }
 
-    if (scrolling && scrollRefresh) {
-      size_t recentPos=currentPos;
-      size_t diff;
-
-      if (currentPos<vAdjustment->GetTop()) {
-        size_t step;
-
-        step=(vAdjustment->GetTop()-currentPos)/STEP;
-        //step=(int)(sqrt(vAdjustment->GetTop()-currentPos)+0.5);
-
-        if (step==0) {
-          step=1;
-        }
-
-        currentPos+=step;
-      }
-      else {
-        size_t step;
-
-        step=(currentPos-vAdjustment->GetTop())/STEP;
-        //step=(int)(sqrt(currentPos-vAdjustment->GetTop())+0.5);
-
-        if (step==0) {
-          step=1;
-        }
-
-        currentPos-=step;
-      }
-
-      diff=std::max(recentPos,currentPos)-std::min(recentPos,currentPos);
+    if (!fullRefresh && top!=vAdjustment->GetTop()) {
+      size_t diff=std::max(top,vAdjustment->GetTop())-std::min(top,vAdjustment->GetTop());
 
       if (diff>=height) {
         DrawFull(draw);
       }
-      else if (currentPos>recentPos) {
+      else if (vAdjustment->GetTop()>top) {
         draw->CopyArea(this->x,this->y+diff,width,height-diff,this->x,this->y);
         DrawRect(draw,height-diff,diff);
       }
-      else if (recentPos>currentPos) {
+      else if (top>vAdjustment->GetTop()) {
         draw->CopyArea(this->x,this->y,width,height-diff,this->x,this->y+diff);
         DrawRect(draw,0,diff);
       }
-
-      scrolling=(currentPos!=vAdjustment->GetTop());
-
-      if (scrolling) {
-        Lum::OS::display->AddTimer(0,1000000/25,scrollTimerAction);
-      }
-      else {
-        scrollRefresh=false;
-      }
     }
     else {
-      scrollRefresh=false;
       DrawRect(draw,y-this->y,h);
     }
+
+    fullRefresh=false;
+    top=vAdjustment->GetTop();
   }
 
   void TableView::Hide()
   {
-    scrollRefresh=false;
-    scrolling=false;
-
     Scrollable::Hide();
   }
 
@@ -1159,79 +1106,35 @@ namespace Lum {
 
     if (event.type==OS::MouseEvent::down && PointIsIn(event)) {
       if (event.button==OS::MouseEvent::button1) {
-        if (GetCell(event.x,event.y,xc,yc) && selection.Valid() && yc<=model->GetRows()) {
-          px=event.x;
-          py=event.y;
-          ppos=currentPos;
+        if (GetCell(event.x,event.y,xc,yc) &&
+            selection.Valid() &&
+            yc<=model->GetRows()) {
 
           selection->SelectCell(xc,yc);
           OnMouseSelection();
           if (GetWindow()->IsDoubleClicked()) {
-            ActionDoubleClick();
+            size_t xl,yl;
+
+            if (PointIsIn(GetWindow()->GetLastButtonClickEvent()) &&
+                GetCell(GetWindow()->GetLastButtonClickEvent().x,
+                        GetWindow()->GetLastButtonClickEvent().y,
+                        xl,yl) &&
+                yl<=model->GetRows() && yc==yl) {
+
+              ActionDoubleClick();
+            }
           }
 
-          my=event.y;
-          sy=event.y;
-          mouseTimer.SetToNow();
-          Lum::OS::display->AddTimer(0,1000000/20,mouseTimerAction);
           return true;
         }
       }
       else if (event.button==OS::MouseEvent::button4) {
-        scrollRefresh=false;
         vAdjustment->DecTop();
       }
       else if (event.button==OS::MouseEvent::button5) {
-        scrollRefresh=false;
         vAdjustment->IncTop();
       }
     }
-    else if (event.type==OS::MouseEvent::move &&
-             (event.qualifier & OS::MouseEvent::button1) &&
-             event.IsGrabed()) {
-      my=event.y;
-
-      if (event.y>=py && (size_t)(event.y-py)>=ppos) {
-        vAdjustment->SetTop(1);
-      }
-      else {
-        vAdjustment->SetTop(ppos+(py-event.y));
-      }
-
-      return true;
-    }
-    else if (event.IsGrabEnd()) {
-      Lum::OS::display->RemoveTimer(mouseTimerAction);
-
-      Lum::Base::SystemTime now;
-
-      now.Sub(mouseTimer);
-
-      // Happens for unknown reason sometimes under windows!
-      if (now.GetTime()==0 && now.GetMicroseconds()==0) {
-        return true;
-      }
-
-      int step=((sy-event.y)*1000000)/(now.GetMicroseconds()+1000000*now.GetTime());
-
-      if (abs(step)<5) {
-        return true;
-      }
-
-      if (step>0) {
-        scrollRefresh=true;
-        vAdjustment->SetTop(vAdjustment->GetTop()+step);
-      }
-      else if (step<0) {
-        scrollRefresh=true;
-        if ((size_t)abs(step)<vAdjustment->GetTop()-1) {
-          vAdjustment->SetTop(vAdjustment->GetTop()+step);
-        }
-        else {
-          vAdjustment->SetTop(1);
-        }
-      }
-    }
 
     return false;
   }
@@ -1256,7 +1159,7 @@ namespace Lum {
       RedrawLine(refreshRow->row);
     }
     else if ((insertRow=dynamic_cast<const Model::Table::InsertRow*>(&msg))!=NULL) {
-      scrollRefresh=false;
+      fullRefresh=true;
       Redraw(); // TODO: Better calculate to be redrawn region...
     }
     else if ((deleteRow=dynamic_cast<const Model::Table::DeleteRow*>(&msg))!=NULL) {
@@ -1265,11 +1168,10 @@ namespace Lum {
           selection->OnLineDeleted(r);
         }
       }
-      scrollRefresh=false;
+      fullRefresh=true;
       Redraw(); // TODO: Better calculate to be redrawn region...
     }
     else if ((resizedColumn=dynamic_cast<const Model::Header::ResizedColumn*>(&msg))!=NULL) {
-      scrollRefresh=false;
       Redraw(); // TODO: Calculate rectangle to get redrawn...
     }
     else if ((fitColumn=dynamic_cast<const Model::Header::FitColumn*>(&msg))!=NULL) {
@@ -1287,29 +1189,20 @@ namespace Lum {
           selection->OnClear();
         }
 
-        scrollRefresh=false;
-        currentPos=1; // We don't want to scroll on a complete refresh...
+        fullRefresh=true;
         vAdjustment->SetTop(1);
-        scrollRefresh=false;
         Redraw();
       }
       else if (model==vAdjustment->GetTopModel()) {
-        if (!scrollRefresh) {
-          currentPos=vAdjustment->GetTop();
-        }
         Redraw();
       }
-      else if (model==hAdjustment->GetTopModel() || model==selection) {
+      else if (model==hAdjustment->GetTopModel()) {
+        fullRefresh=true;
         Redraw();
       }
-      else if (model==scrollTimerAction && scrollTimerAction->IsFinished()) {
+      else if (model==selection) {
         Redraw();
       }
-      else if (model==mouseTimerAction && mouseTimerAction->IsFinished()) {
-        sy=my;
-        mouseTimer.SetToNow();
-        Lum::OS::display->AddTimer(0,1000000/20,mouseTimerAction);
-      }
     }
 
     Scrollable::Resync(model,msg);
