@@ -24,6 +24,7 @@
 #include <Lum/Base/String.h>
 #include <Lum/Base/Util.h>
 
+#include <Lum/OS/Behaviour.h>
 #include <Lum/OS/Driver.h>
 #include <Lum/OS/Theme.h>
 
@@ -122,6 +123,8 @@ namespace Lum {
 
   void Window::KeyHandler::SetFocus(Lum::Object* object)
   {
+    Lum::Object* oldCurrent=current;
+
     if (current==object) {
       return;
     }
@@ -137,7 +140,7 @@ namespace Lum {
     }
 
     if (OS::display->GetKeyboardManager()!=NULL) {
-      OS::display->GetKeyboardManager()->OnFocusChange(window,current,object);
+      OS::display->GetKeyboardManager()->OnFocusChange(window,oldCurrent,object);
     }
   }
 
@@ -157,19 +160,17 @@ namespace Lum {
 
     if (current!=NULL) {
       object=current->GetFocusNext(current);
-      if (object!=NULL) {
-        SetFocus(object);
-        return;
-      }
+      SetFocus(object);
+      return;
     }
 
     object=top->GetFocusFirst();
-    if (object!=NULL && object!=current) {
+    if (object!=current) {
       SetFocus(object);
     }
   }
 
-  void Window::KeyHandler::SetFocusLast()
+  void Window::KeyHandler::SetFocusPrevious()
   {
     Lum::Object *object;
 
@@ -194,20 +195,22 @@ namespace Lum {
   {
   }
 
-  void Window::KeyHandler::SetFocusLastInGroup()
+  void Window::KeyHandler::SetFocusPreviousInGroup()
   {
   }
 
   void Window::KeyHandler::Activate()
   {
-    if (current!=NULL && current->HandlesFocus()) {
-      if (OS::display->GetKeyboardManager()!=NULL) {
-        OS::display->GetKeyboardManager()->OnFocusChange(window,current,current);
+    if (current!=NULL ||Lum::OS::display->GetBehaviour()->FocusObjectOnWindowFocusIn()) {
+      if (current!=NULL && current->HandlesFocus()) {
+        if (OS::display->GetKeyboardManager()!=NULL) {
+          OS::display->GetKeyboardManager()->OnFocusChange(window,current,current);
+        }
+        current->RecatchedFocus();
+      }
+      else {
+        SetFocusFirst();
       }
-      current->RecatchedFocus();
-    }
-    else {
-      SetFocusFirst();
     }
   }
 
@@ -339,19 +342,21 @@ namespace Lum {
         }
         break;
       case OS::keyLeftTab:
-        SetFocusLast();
+        SetFocusPrevious();
         return true;
         break;
       case OS::keyLeft:
       case OS::keyUp:
         if (event->qualifier==0) {
-          SetFocusLastInGroup();
+          SetFocusPrevious();
+          //SetFocusPreviousInGroup();
         }
         break;
       case OS::keyRight:
       case OS::keyDown:
         if (event->qualifier==0) {
-          SetFocusNextInGroup();
+          SetFocusNext();
+          //SetFocusNextInGroup();
         }
         break;
       default:
@@ -396,6 +401,10 @@ namespace Lum {
   {
     deleted=true;
 
+    if (window->IsOpen()) {
+      window->Close();
+    }
+
     delete main;
     delete window;
   }
@@ -646,13 +655,19 @@ namespace Lum {
 
       bool Visit(Lum::Object* object)
       {
+        if (object->InterceptMouseEvents() &&
+            object->HandleMouseEvent(event)) {
+          grab=object;
+        }
+
         object->VisitChildren(*this,true);
 
         if (grab!=NULL) {
           return false;
         }
 
-        if (object->HandleMouseEvent(event)) {
+        if (!object->InterceptMouseEvents() &&
+            object->HandleMouseEvent(event)) {
           grab=object;
         }
 
@@ -685,7 +700,16 @@ namespace Lum {
       exists, to the top most object
     */
     if ((mouseEvent=dynamic_cast<OS::MouseEvent*>(event))!=NULL) {
-      if (mouseEvent->type==OS::MouseEvent::down && mouseEvent->button==Lum::OS::MouseEvent::button3) {
+      /*
+        Mouse grabbing and/or synchronous subdialogs may lead to mouse
+        and key event occuring after window has been hidden!
+      */
+      if (!IsOpen() || !IsShown()) {
+        return false;
+      }
+
+      if (mouseEvent->type==OS::MouseEvent::down &&
+          mouseEvent->button==Lum::OS::MouseEvent::button3) {
         if (OnContextMenu()) {
           // menu mouse button up might have been lost, we kill the current mouse grab
           // to be on the save side.
@@ -720,6 +744,15 @@ namespace Lum {
         mouseGrab->HandleMouseEvent(*mouseEvent);
 
         if (mouseEvent->IsGrabEnd()) {
+          if (mouseEvent->type==OS::MouseEvent::up &&
+              mouseEvent->button==Lum::OS::MouseEvent::button1 &&
+              mouseEvent->qualifier==Lum::OS::MouseEvent::button1 &&
+              OS::display->GetKeyboardManager()!=NULL) {
+            OS::display->GetKeyboardManager()->OnMouseClick(window,
+                                                            mouseGrab,
+                                                            mouseEvent);
+          }
+
           mouseGrab=NULL;
         }
       }
@@ -735,7 +768,8 @@ namespace Lum {
 
         mouseGrabHunting=false;
 
-        if (mouseGrab!=NULL && mouseGrab->HandlesFocus()) {
+        if (mouseGrab!=NULL &&
+            mouseGrab->HandlesFocus()) {
           SetFocus(dynamic_cast<Lum::Object*>(mouseGrab));
         }
       }
@@ -749,6 +783,14 @@ namespace Lum {
       EvaluateMouseActive();
     }
     else if ((keyEvent=dynamic_cast<OS::KeyEvent*>(event))!=NULL) {
+      /*
+        Mouse grabbing and/or synchronous subdialogs may lead to mouse
+        and key event occuring after window has been hidden!
+      */
+      if (!IsShown()) {
+        return false;
+      }
+
       keyHandler.HandleEvent(keyEvent);
     }
 
@@ -794,7 +836,8 @@ namespace Lum {
 
   void Window::Resync(Base::Model* model, const Base::ResyncMsg& msg)
   {
-    if (model==window->GetUnmapedAction() && window->GetUnmapedAction()->IsFinished()) {
+    if (model==window->GetUnmapedAction() &&
+        window->GetUnmapedAction()->IsFinished()) {
       OS::display->RemoveTimer(menuAction);
 
       if (mouseActive!=NULL) {
@@ -805,12 +848,14 @@ namespace Lum {
       mouseGrab=NULL;
       main->Hide();
     }
-    else if (model==window->GetFocusInAction() && window->GetFocusInAction()->IsFinished()) {
+    else if (model==window->GetFocusInAction() &&
+             window->GetFocusInAction()->IsFinished()) {
       mouseGrab=NULL;
       keyHandler.Activate();
       EvaluateMouseActive();
     }
-    else if (model==window->GetFocusOutAction() && window->GetFocusOutAction()->IsFinished()) {
+    else if (model==window->GetFocusOutAction() &&
+             window->GetFocusOutAction()->IsFinished()) {
       OS::display->RemoveTimer(menuAction);
 
       mouseGrab=NULL;
@@ -818,10 +863,12 @@ namespace Lum {
       keyHandler.Deactivate();
       EvaluateMouseActive();
     }
-    else if (model==window->GetMouseInAction() && window->GetMouseInAction()->IsFinished()) {
+    else if (model==window->GetMouseInAction() &&
+             window->GetMouseInAction()->IsFinished()) {
       EvaluateMouseActive();
     }
-    else if (model==window->GetMouseOutAction() && window->GetMouseOutAction()->IsFinished()) {
+    else if (model==window->GetMouseOutAction() &&
+             window->GetMouseOutAction()->IsFinished()) {
       OS::display->RemoveTimer(menuAction);
 
       if (mouseActive!=NULL) {
@@ -829,13 +876,15 @@ namespace Lum {
         mouseActive=NULL;
       }
     }
-    else if (model==window->GetResizeAction() && window->GetResizeAction()->IsFinished()) {
+    else if (model==window->GetResizeAction() &&
+             window->GetResizeAction()->IsFinished()) {
       const OS::Window::ResizeMsg *resizeMsg=dynamic_cast<const OS::Window::ResizeMsg*>(&msg);
 
       main->Resize(resizeMsg->width,resizeMsg->height);
       EvaluateMouseActive();
     }
-    else if (model==window->GetRedrawAction() && window->GetRedrawAction()->IsFinished()) {
+    else if (model==window->GetRedrawAction() &&
+             window->GetRedrawAction()->IsFinished()) {
       const OS::Window::RedrawMsg *redrawMsg=dynamic_cast<const OS::Window::RedrawMsg*>(&msg);
 
       main->Move(0,0);
@@ -843,27 +892,32 @@ namespace Lum {
       main->DrawBackground(window->GetDrawInfo(),redrawMsg->x,redrawMsg->y,redrawMsg->width,redrawMsg->height);
       main->Draw(redrawMsg->x,redrawMsg->y,redrawMsg->width,redrawMsg->height);
     }
-    else if (model==window->GetPreInitAction() && window->GetPreInitAction()->IsFinished()) {
+    else if (model==window->GetPreInitAction() &&
+             window->GetPreInitAction()->IsFinished()) {
       PreInit();
 
       CalcSize();
     }
-    else if (model==window->GetOpenedAction() && window->GetOpenedAction()->IsFinished()) {
+    else if (model==window->GetOpenedAction() &&
+             window->GetOpenedAction()->IsFinished()) {
       EvaluateMouseActive();
     }
-    else if (model==window->GetEventAction() && window->GetEventAction()->IsFinished()) {
+    else if (model==window->GetEventAction() &&
+             window->GetEventAction()->IsFinished()) {
       const OS::Window::EventMsg *eventMsg=dynamic_cast<const OS::Window::EventMsg*>(&msg);
 
       HandleEvent(eventMsg->event);
     }
-    else if (model==menuAction && menuAction->IsFinished()) {
+    else if (model==menuAction &&
+             menuAction->IsFinished()) {
       if (OnContextMenu()) {
         // menu mouse button up might have been lost, we kill the current mouse grab
         // to be on the save side.
         mouseGrab=NULL;
       }
     }
-    else if (model==window->GetGrabCancelAction() && window->GetGrabCancelAction()->IsFinished()) {
+    else if (model==window->GetGrabCancelAction() &&
+             window->GetGrabCancelAction()->IsFinished()) {
       // A child popup opened while we were handling mouse button grab events.
       OS::display->RemoveTimer(menuAction);
 
