@@ -103,28 +103,20 @@ namespace Lum {
 
   bool WindowGroup::VisitChildren(Visitor &visitor, bool /*onlyVisible*/)
   {
-    if (men!=NULL) {
-      if (!visitor.Visit(men)) {
-        return false;
-      }
+    if (men!=NULL && !visitor.Visit(men)) {
+      return false;
     }
 
-    if (main!=NULL) {
-      if (!visitor.Visit(main)) {
-        return false;
-      }
+    if (main!=NULL && !visitor.Visit(main)) {
+      return false;
     }
 
-    if (tool!=NULL) {
-      if (!visitor.Visit(tool)) {
-        return false;
-      }
+    if (tool!=NULL && !visitor.Visit(tool)) {
+      return false;
     }
 
-    if (status!=NULL) {
-      if (!visitor.Visit(status)) {
-        return false;
-      }
+    if (status!=NULL && !visitor.Visit(status)) {
+      return false;
     }
 
     return true;
