@@ -92,32 +92,72 @@ namespace Lum {
       panel->Add(title);
       panel->AddSpace();
       panel->Add(pages[i].content);
-      panel->AddSpace();
 
-      row=ButtonRow::Create(true,false);
+      if (OS::display->GetTheme()->GetDialogActionPosition()==OS::Theme::actionPosHorizButtonRowBelow) {
+        panel->AddSpace();
 
-      if (pages[i].helpAction.Valid()) {
-        row->AddOptional(Button::CreateHelp(pages[i].helpAction,true,true));
+        row=ButtonRow::Create(true,false);
+
+        if (pages[i].helpAction.Valid()) {
+          row->AddOptional(Button::CreateHelp(pages[i].helpAction,true,true));
+        }
+
+        if (i>0) {
+          row->Add(Button::Create(L"_Previous",backAction,true,true));
+        }
+
+        if (i==pages.size()-1) {
+          row->Add(Button::Create(L"_Finish",pages[i].continueAction,true,true));
+        }
+        else {
+          row->Add(Button::Create(L"_Next",pages[i].continueAction,true,true));
+        }
+
+        if (cancelAction.Valid()) {
+          row->Add(Button::CreateCancel(cancelAction,true,true));
+        }
+
+        panel->Add(row);
+
+        multi->Append(panel);
       }
+      else if (OS::display->GetTheme()->GetDialogActionPosition()==OS::Theme::actionPosVertButtonRowRight) {
+        HPanel *main;
+        VPanel *buttons;
 
-      if (i>0) {
-        row->Add(Button::Create(L"_Previous",backAction,true,true));
-      }
+        main=HPanel::Create(true,true);
+        main->Add(panel);
+        main->AddSpace();
 
-      if (i==pages.size()-1) {
-        row->Add(Button::Create(L"_Finish",pages[i].continueAction,true,true));
+        buttons=VPanel::Create(false,true);
+        buttons->AddSpace(true);
+
+        if (pages[i].helpAction.Valid()) {
+          buttons->Add(Button::CreateHelp(pages[i].helpAction,true,false));
+        }
+
+        if (i>0) {
+          buttons->Add(Button::Create(L"_Previous",backAction,true,false));
+        }
+
+        if (i==pages.size()-1) {
+          buttons->Add(Button::Create(L"_Finish",pages[i].continueAction,true,false));
+        }
+        else {
+          buttons->Add(Button::Create(L"_Next",pages[i].continueAction,true,false));
+        }
+
+        if (cancelAction.Valid()) {
+          buttons->Add(Button::CreateCancel(cancelAction,true,false));
+        }
+
+        main->Add(buttons);
+
+        multi->Append(main);
       }
       else {
-        row->Add(Button::Create(L"_Next",pages[i].continueAction,true,true));
+        assert(false);
       }
-
-      if (cancelAction.Valid()) {
-        row->Add(Button::CreateCancel(cancelAction,true,true));
-      }
-
-      panel->Add(row);
-
-      multi->Append(panel);
     }
 
     container=multi;
